{ lib, pkgs, config,  ... }:
let
  secrets = config.myEnv.websites.connexionswing.integration;
  webRoot = "/var/lib/ftp/immae/connexionswing/web";
  varDir = "/var/lib/ftp/immae/connexionswing_var";
  cfg = config.myServices.websites.connexionswing.integration;
  pcfg = config.services.phpApplication;
in {
  options.myServices.websites.connexionswing.integration.enable = lib.mkEnableOption "enable Connexionswing's website in integration";

  config = lib.mkIf cfg.enable {
    services.phpApplication.apps.connexionswing_integration = {
      websiteEnv = "integration";
      httpdUser = config.services.httpd.Inte.user;
      httpdGroup = config.services.httpd.Inte.group;
      inherit webRoot varDir;
      varDirPaths = {
        "medias" = "0700";
        "uploads" = "0700";
        "var" = "0700";
      };
      app = "/var/lib/ftp/immae/connexionswing";
      serviceDeps = [ "mysql.service" ];
      preStartActions = [
        "./bin/console --env=dev cache:clear --no-warmup"
      ];
      phpOpenbasedir = [ "/tmp" "/run/wrappers/bin/sendmail" ];
      phpPool = {
        "php_admin_value[upload_max_filesize]" = "20M";
        "php_admin_value[post_max_size]" = "20M";
        #"php_admin_flag[log_errors]" = "on";
        "pm" = "ondemand";
        "pm.max_children" = "5";
        "pm.process_idle_timeout" = "60";
        };
      phpEnv = {
        SYMFONY_DEBUG_MODE = "\"yes\"";
      };
      phpWatchFiles = [
        config.secrets.fullPaths."websites/connexionswing/integration"
      ];
      phpPackage = pkgs.php72;
    };

    secrets.keys."websites/connexionswing/integration" = {
      user = config.services.httpd.Inte.user;
      group = config.services.httpd.Inte.group;
      permissions = "0400";
      text = ''
        # This file is auto-generated during the composer install
        parameters:
            database_host: ${secrets.mysql.host}
            database_port: ${secrets.mysql.port}
            database_name: ${secrets.mysql.database}
            database_user: ${secrets.mysql.user}
            database_password: ${secrets.mysql.password}
            database_server_version: ${pkgs.mariadb.mysqlVersion}
            mailer_transport: sendmail
            mailer_host: null
            mailer_user: null
            mailer_password: null
            subscription_email: ${secrets.email}
            allow_robots: true
            secret: ${secrets.secret}
      '';
    };

    services.websites.env.integration.vhostConfs.connexionswing_integration = {
      certName    = "integration";
      addToCerts  = true;
      hosts       = ["connexionswing.immae.eu" "sandetludo.immae.eu" ];
      root        = webRoot;
      extraConfig = [
        ''
        <FilesMatch "\.php$">
          SetHandler "proxy:unix:${pcfg.phpListenPaths.connexionswing_integration}|fcgi://localhost"
        </FilesMatch>

        <Directory ${varDir}/medias>
          Options FollowSymLinks
          AllowOverride None
          Require all granted
        </Directory>

        <Directory ${varDir}/uploads>
          Options FollowSymLinks
          AllowOverride None
          Require all granted
        </Directory>

        <Location />
          Use LDAPConnect
          Require ldap-group   cn=connexionswing.immae.eu,cn=httpd,ou=services,dc=immae,dc=eu
          ErrorDocument 401 "<html><meta http-equiv=\"refresh\" content=\"0;url=https://connexionswing.com\"></html>"
        </Location>

        <Directory ${webRoot}>
          Options Indexes FollowSymLinks MultiViews Includes
          AllowOverride None
          Require all granted

          DirectoryIndex app_dev.php

          <IfModule mod_negotiation.c>
          Options -MultiViews
          </IfModule>

          <IfModule mod_rewrite.c>
            RewriteEngine On

            RewriteCond %{REQUEST_URI}::$1 ^(/.+)/(.*)::\2$
            RewriteRule ^(.*) - [E=BASE:%1]

            # Maintenance script
            RewriteCond %{DOCUMENT_ROOT}/maintenance.php -f
            RewriteCond %{SCRIPT_FILENAME} !maintenance.php
            RewriteRule ^.*$ %{ENV:BASE}/maintenance.php [R=503,L]
            ErrorDocument 503 /maintenance.php

            # Sets the HTTP_AUTHORIZATION header removed by Apache
            RewriteCond %{HTTP:Authorization} .
            RewriteRule ^ - [E=HTTP_AUTHORIZATION:%{HTTP:Authorization}]

            RewriteCond %{ENV:REDIRECT_STATUS} ^$
            RewriteRule ^app_dev\.php(?:/(.*)|$) %{ENV:BASE}/$1 [R=301,L]

            # If the requested filename exists, simply serve it.
            # We only want to let Apache serve files and not directories.
            RewriteCond %{REQUEST_FILENAME} -f
            RewriteRule ^ - [L]

            # Rewrite all other queries to the front controller.
            RewriteRule ^ %{ENV:BASE}/app_dev.php [L]
          </IfModule>

        </Directory>
        ''
      ];
    };
  };
}
